@@ -1,21 +1,12 @@
 /*
- * Copyright (C) 2004, 2005, 2007, 2009  Internet Systems Consortium, Inc. ("ISC")
- * Copyright (C) 1999-2003  Internet Software Consortium.
+ * Copyright (C) 1999-2005, 2007, 2009, 2015, 2016  Internet Systems Consortium, Inc. ("ISC")
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-/* $Id: loc_29.c,v 1.50 2009-12-04 21:09:33 marka Exp $ */
+/* $Id: loc_29.c,v 1.50 2009/12/04 21:09:33 marka Exp $ */
 
 /* Reviewed: Wed Mar 15 18:13:09 PST 2000 by explorer */
 
@@ -50,7 +41,7 @@ fromtext_loc(ARGS_FROMTEXT) {
 	unsigned long longitude;
 	unsigned long altitude;
 
-	REQUIRE(type == 29);
+	REQUIRE(type == dns_rdatatype_loc);
 
 	UNUSED(type);
 	UNUSED(rdclass);
@@ -475,12 +466,16 @@ totext_loc(ARGS_TOTEXT) {
 
 	UNUSED(tctx);
 
-	REQUIRE(rdata->type == 29);
+	REQUIRE(rdata->type == dns_rdatatype_loc);
 	REQUIRE(rdata->length != 0);
 
 	dns_rdata_toregion(rdata, &sr);
 
-	/* version = sr.base[0]; */
+	if (sr.base[0] != 0)
+		return (ISC_R_NOTIMPLEMENTED);
+
+	REQUIRE(rdata->length == 16);
+
 	size = sr.base[1];
 	INSIST((size&0x0f) < 10 && (size>>4) < 10);
 	if ((size&0x0f)> 1)
@@ -563,7 +558,7 @@ fromwire_loc(ARGS_FROMWIRE) {
 	unsigned long latitude;
 	unsigned long longitude;
 
-	REQUIRE(type == 29);
+	REQUIRE(type == dns_rdatatype_loc);
 
 	UNUSED(type);
 	UNUSED(rdclass);
@@ -573,8 +568,11 @@ fromwire_loc(ARGS_FROMWIRE) {
 	isc_buffer_activeregion(source, &sr);
 	if (sr.length < 1)
 		return (ISC_R_UNEXPECTEDEND);
-	if (sr.base[0] != 0)
-		return (ISC_R_NOTIMPLEMENTED);
+	if (sr.base[0] != 0) {
+		/* Treat as unknown. */
+		isc_buffer_forward(source, sr.length);
+		return (mem_tobuffer(target, sr.base, sr.length));
+	}
 	if (sr.length < 16)
 		return (ISC_R_UNEXPECTEDEND);
 
@@ -634,7 +632,7 @@ static inline isc_result_t
 towire_loc(ARGS_TOWIRE) {
 	UNUSED(cctx);
 
-	REQUIRE(rdata->type == 29);
+	REQUIRE(rdata->type == dns_rdatatype_loc);
 	REQUIRE(rdata->length != 0);
 
 	return (mem_tobuffer(target, rdata->data, rdata->length));
@@ -647,7 +645,7 @@ compare_loc(ARGS_COMPARE) {
 
 	REQUIRE(rdata1->type == rdata2->type);
 	REQUIRE(rdata1->rdclass == rdata2->rdclass);
-	REQUIRE(rdata1->type == 29);
+	REQUIRE(rdata1->type == dns_rdatatype_loc);
 	REQUIRE(rdata1->length != 0);
 	REQUIRE(rdata2->length != 0);
 
@@ -661,7 +659,7 @@ fromstruct_loc(ARGS_FROMSTRUCT) {
 	dns_rdata_loc_t *loc = source;
 	isc_uint8_t c;
 
-	REQUIRE(type == 29);
+	REQUIRE(type == dns_rdatatype_loc);
 	REQUIRE(source != NULL);
 	REQUIRE(loc->common.rdtype == type);
 	REQUIRE(loc->common.rdclass == rdclass);
@@ -706,7 +704,7 @@ tostruct_loc(ARGS_TOSTRUCT) {
 	isc_region_t r;
 	isc_uint8_t version;
 
-	REQUIRE(rdata->type == 29);
+	REQUIRE(rdata->type == dns_rdatatype_loc);
 	REQUIRE(target != NULL);
 	REQUIRE(rdata->length != 0);
 
@@ -743,7 +741,7 @@ freestruct_loc(ARGS_FREESTRUCT) {
 	dns_rdata_loc_t *loc = source;
 
 	REQUIRE(source != NULL);
-	REQUIRE(loc->common.rdtype == 29);
+	REQUIRE(loc->common.rdtype == dns_rdatatype_loc);
 
 	UNUSED(source);
 	UNUSED(loc);
@@ -751,7 +749,7 @@ freestruct_loc(ARGS_FREESTRUCT) {
 
 static inline isc_result_t
 additionaldata_loc(ARGS_ADDLDATA) {
-	REQUIRE(rdata->type == 29);
+	REQUIRE(rdata->type == dns_rdatatype_loc);
 
 	UNUSED(rdata);
 	UNUSED(add);
@@ -764,7 +762,7 @@ static inline isc_result_t
 digest_loc(ARGS_DIGEST) {
 	isc_region_t r;
 
-	REQUIRE(rdata->type == 29);
+	REQUIRE(rdata->type == dns_rdatatype_loc);
 
 	dns_rdata_toregion(rdata, &r);
 
@@ -774,7 +772,7 @@ digest_loc(ARGS_DIGEST) {
 static inline isc_boolean_t
 checkowner_loc(ARGS_CHECKOWNER) {
 
-	REQUIRE(type == 29);
+	REQUIRE(type == dns_rdatatype_loc);
 
 	UNUSED(name);
 	UNUSED(type);
@@ -787,7 +785,7 @@ checkowner_loc(ARGS_CHECKOWNER) {
 static inline isc_boolean_t
 checknames_loc(ARGS_CHECKNAMES) {
 
-	REQUIRE(rdata->type == 29);
+	REQUIRE(rdata->type == dns_rdatatype_loc);
 
 	UNUSED(rdata);
 	UNUSED(owner);
