@@ -1,17 +1,9 @@
 /*
- * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2000, 2001, 2003-2008, 2010, 2011, 2013, 2014, 2016  Internet Systems Consortium, Inc. ("ISC")
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
 /*! \file */
@@ -347,20 +339,9 @@ stf_from_address(dns_name_t *stfself, isc_netaddr_t *tcpaddr) {
 
 isc_boolean_t
 dns_ssutable_checkrules(dns_ssutable_t *table, dns_name_t *signer,
-			dns_name_t *name, isc_netaddr_t *addr,
-			dns_rdatatype_t type, const dst_key_t *key)
-{
-	return (dns_ssutable_checkrules2
-		(table, signer, name, addr,
-		 addr == NULL ? ISC_FALSE : ISC_TRUE,
-		 NULL, type, key));
-}
-
-isc_boolean_t
-dns_ssutable_checkrules2(dns_ssutable_t *table, dns_name_t *signer,
-			 dns_name_t *name, isc_netaddr_t *addr,
-			 isc_boolean_t tcp, const dns_aclenv_t *env,
-			 dns_rdatatype_t type, const dst_key_t *key)
+			dns_name_t *name, isc_netaddr_t *tcpaddr,
+			dns_rdatatype_t type,
+			const dst_key_t *key)
 {
 	dns_ssurule_t *rule;
 	unsigned int i;
@@ -369,14 +350,12 @@ dns_ssutable_checkrules2(dns_ssutable_t *table, dns_name_t *signer,
 	dns_name_t *tcpself;
 	dns_name_t *stfself;
 	isc_result_t result;
-	int match;
 
 	REQUIRE(VALID_SSUTABLE(table));
 	REQUIRE(signer == NULL || dns_name_isabsolute(signer));
 	REQUIRE(dns_name_isabsolute(name));
-	REQUIRE(addr == NULL || env != NULL);
 
-	if (signer == NULL && addr == NULL)
+	if (signer == NULL && tcpaddr == NULL)
 		return (ISC_FALSE);
 
 	for (rule = ISC_LIST_HEAD(table->rules);
@@ -385,7 +364,6 @@ dns_ssutable_checkrules2(dns_ssutable_t *table, dns_name_t *signer,
 	{
 		switch (rule->matchtype) {
 		case DNS_SSUMATCHTYPE_NAME:
-		case DNS_SSUMATCHTYPE_LOCAL:
 		case DNS_SSUMATCHTYPE_SUBDOMAIN:
 		case DNS_SSUMATCHTYPE_WILDCARD:
 		case DNS_SSUMATCHTYPE_SELF:
@@ -411,7 +389,7 @@ dns_ssutable_checkrules2(dns_ssutable_t *table, dns_name_t *signer,
 			break;
 		case DNS_SSUMATCHTYPE_TCPSELF:
 		case DNS_SSUMATCHTYPE_6TO4SELF:
-			if (!tcp || addr == NULL)
+			if (tcpaddr == NULL)
 				continue;
 			break;
 		}
@@ -425,29 +403,6 @@ dns_ssutable_checkrules2(dns_ssutable_t *table, dns_name_t *signer,
 			if (!dns_name_issubdomain(name, rule->name))
 				continue;
 			break;
-		case DNS_SSUMATCHTYPE_LOCAL:
-			if (addr == NULL) {
-				continue;
-			}
-			if (!dns_name_issubdomain(name, rule->name)) {
-				continue;
-			}
-			dns_acl_match(addr, NULL, env->localhost,
-				      NULL, &match, NULL);
-			if (match == 0) {
-				if (signer != NULL) {
-					isc_log_write(dns_lctx,
-						      DNS_LOGCATEGORY_GENERAL,
-						      DNS_LOGMODULE_SSU,
-						      ISC_LOG_WARNING,
-						      "update-policy local: "
-						      "match on session "
-						      "key not from "
-						      "localhost");
-				}
-				continue;
-			}
-			break;
 		case DNS_SSUMATCHTYPE_WILDCARD:
 			if (!dns_name_matcheswildcard(name, rule->name))
 				continue;
@@ -497,7 +452,7 @@ dns_ssutable_checkrules2(dns_ssutable_t *table, dns_name_t *signer,
 		case DNS_SSUMATCHTYPE_TCPSELF:
 			dns_fixedname_init(&fixed);
 			tcpself = dns_fixedname_name(&fixed);
-			reverse_from_address(tcpself, addr);
+			reverse_from_address(tcpself, tcpaddr);
 			if (dns_name_iswildcard(rule->identity)) {
 				if (!dns_name_matcheswildcard(tcpself,
 							      rule->identity))
@@ -512,7 +467,7 @@ dns_ssutable_checkrules2(dns_ssutable_t *table, dns_name_t *signer,
 		case DNS_SSUMATCHTYPE_6TO4SELF:
 			dns_fixedname_init(&fixed);
 			stfself = dns_fixedname_name(&fixed);
-			stf_from_address(stfself, addr);
+			stf_from_address(stfself, tcpaddr);
 			if (dns_name_iswildcard(rule->identity)) {
 				if (!dns_name_matcheswildcard(stfself,
 							      rule->identity))
@@ -526,13 +481,13 @@ dns_ssutable_checkrules2(dns_ssutable_t *table, dns_name_t *signer,
 			break;
 		case DNS_SSUMATCHTYPE_EXTERNAL:
 			if (!dns_ssu_external_match(rule->identity, signer,
-						    name, addr, type, key,
+						    name, tcpaddr, type, key,
 						    table->mctx))
 				continue;
 			break;
 		case DNS_SSUMATCHTYPE_DLZ:
 			if (!dns_dlz_ssumatch(table->dlzdatabase, signer,
-					      name, addr, type, key))
+					      name, tcpaddr, type, key))
 				continue;
 			break;
 		}
@@ -647,43 +602,3 @@ dns_ssutable_createdlz(isc_mem_t *mctx, dns_ssutable_t **tablep,
 	*tablep = table;
 	return (ISC_R_SUCCESS);
 }
-
-isc_result_t
-dns_ssu_mtypefromstring(const char *str, dns_ssumatchtype_t *mtype) {
-
-	REQUIRE(str != NULL);
-	REQUIRE(mtype != NULL);
-
-	if (strcasecmp(str, "name") == 0) {
-		*mtype = dns_ssumatchtype_name;
-	} else if (strcasecmp(str, "subdomain") == 0) {
-		*mtype = dns_ssumatchtype_subdomain;
-	} else if (strcasecmp(str, "wildcard") == 0) {
-		*mtype = dns_ssumatchtype_wildcard;
-	} else if (strcasecmp(str, "self") == 0) {
-		*mtype = dns_ssumatchtype_self;
-	} else if (strcasecmp(str, "selfsub") == 0) {
-		*mtype = dns_ssumatchtype_selfsub;
-	} else if (strcasecmp(str, "selfwild") == 0) {
-		*mtype = dns_ssumatchtype_selfwild;
-	} else if (strcasecmp(str, "ms-self") == 0) {
-		*mtype = dns_ssumatchtype_selfms;
-	} else if (strcasecmp(str, "krb5-self") == 0) {
-		*mtype = dns_ssumatchtype_selfkrb5;
-	} else if (strcasecmp(str, "ms-subdomain") == 0) {
-		*mtype = dns_ssumatchtype_subdomainms;
-	} else if (strcasecmp(str, "krb5-subdomain") == 0) {
-		*mtype = dns_ssumatchtype_subdomainkrb5;
-	} else if (strcasecmp(str, "tcp-self") == 0) {
-		*mtype = dns_ssumatchtype_tcpself;
-	} else if (strcasecmp(str, "6to4-self") == 0) {
-		*mtype = dns_ssumatchtype_6to4self;
-	} else if (strcasecmp(str, "zonesub") == 0) {
-		*mtype = dns_ssumatchtype_subdomain;
-	} else if (strcasecmp(str, "external") == 0) {
-		*mtype = dns_ssumatchtype_external;
-	} else {
-		return (ISC_R_NOTFOUND);
-	}
-	return (ISC_R_SUCCESS);
-}
